<?php

namespace Drupal\migrate_paragraphs_subprocess\Plugin\migrate\process;

use Drupal\Core\Database\Connection;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\Attribute\MigrateProcess;
use Drupal\migrate\Plugin\migrate\process\SubProcess;
use Drupal\migrate\MigrateExecutableInterface;
use Drupal\migrate\Row;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Process plugin to handle paragraphs sub-process from multiple migrations.
 *
 * This plugin queries migrate map tables directly for
 * entity_reference_revisions mappings so you can migrate paragraphs belonging
 * to different paragraph-type migrations into their parent entity seamlessly.
 */
#[MigrateProcess(
  id: "paragraphs_sub_process",
  handle_multiples: TRUE,
)]
class ParagraphsSubProcess extends SubProcess implements ContainerFactoryPluginInterface {

  /**
   * Constructs a new ParagraphsSubProcess plugin.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection service.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    $plugin_definition,
    protected readonly Connection $database
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   *
   * Transforms source paragraph references into destination paragraph IDs and
   * revision IDs by looking up each configured migration's map table.
   */
  public function transform($value, MigrateExecutableInterface $migrate_executable, Row $row, $destination_property) {
    $return = [];
    // Iterate through each migration ID configured and lookup paragraphs.
    foreach ($this->configuration['migration'] as $migration) {
      $paragraphs = $this->lookupParagraphs($migration, $value);
      // Process each found paragraph sub-process.
      $return += parent::transform($paragraphs, $migrate_executable, $row, $destination_property);
    }

    // Ensure the returned paragraphs are sorted by original delta.
    ksort($return);
    return $return;
  }

  /**
   * Lookup paragraphs by source target_id in the migration map.
   *
   * @param string $migration_id
   *   The migration ID whose map table to query (without prefix).
   * @param array $source_id_values
   *   An array of source ID value arrays keyed by delta:
   *     - 'target_id': the original paragraph entity ID.
   *     - 'target_revision_id': (optional) the original revision ID.
   *
   * @return array
   *   An associative array keyed by delta, where each value is a two-element
   *   array of [destid1 (paragraph ID), destid2 (revision ID)].
   */
  protected function lookupParagraphs(string $migration_id, array $source_id_values): array {
    $paragraphs = [];
    foreach ($source_id_values as $delta => $ids) {
      if (isset($ids['target_id'])) {
        // Query the migrate map table directly.
        $query = $this->database
          ->select("migrate_map_{$migration_id}", 'mm')
          ->fields('mm', ['sourceid1', 'destid1', 'destid2'])
          ->condition('mm.sourceid1', $ids['target_id']);

        $results = $query->execute()->fetchAll();
        foreach ($results as $result) {
          $paragraphs[$delta] = [$result->destid1, $result->destid2];
        }
      }
    }
    return $paragraphs;
  }

}
